local ADDON_NAME, BigBrother = ...

_G.BigBrother = LibStub("AceAddon-3.0"):NewAddon(BigBrother, "BigBrother", "AceEvent-3.0", "AceHook-3.0")
BigBrother:SetDefaultModuleLibraries("AceEvent-3.0")

local L = LibStub("AceLocale-3.0"):GetLocale("BigBrother")

local optionsFrame
local function GetOptions()
	local db = BigBrother.db.profile
	local options = {
		name = ADDON_NAME,
		type = "group",
		childGroups = "tab",
		args = {
			confdesc = {
				name = GetAddOnMetadata("BigBrother", "Notes").."\n",
				type = "description",
				order = 1,
				width = "full",
			},
		},
	}

	for _, module in BigBrother:IterateModules() do
		if module.AddOptions then
			module:AddOptions(options)
		end
	end

	options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(BigBrother.db)
	options.args.profile.desc = nil
	options.args.profile.order = 100

	return options
end

function BigBrother:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("BigBrotherDB", {
		profile = {
			autogroups = true,
			groups = { true, true, true, true, true, false, false, false },
		}
	}, true)

	LibStub("AceConfig-3.0"):RegisterOptionsTable("BigBrother", GetOptions)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("BigBrother")

	--raw hook otherwise SetItemRef will also fire on other |H's in the message
	self:RawHook("SetItemRef", function(link, ...)
		if strsub(link, 1, 10) ~= "bigbrother" then
			return self.hooks["SetItemRef"](link, ...)
		end
		if not IsShiftKeyDown() then return end

		local _, msg = strsplit(":", link, 2)
		msg = msg:gsub("@", "|")
		local editBox = ChatEdit_ChooseBoxForSend()
		if editBox:IsShown() and editBox:GetText() ~= "" then
			editBox:Insert(" "..msg)
		else
			ChatEdit_ActivateChat(editBox)
			editBox:SetText(msg)
		end
	end, true)
end

function BigBrother:OnEnable()
	self:RegisterEvent("CHANNEL_UI_UPDATE")
end

function BigBrother:Print(...)
	print("|cff33ff99BigBrother|r:", tostringall(...))
end

function BigBrother:GetInstanceSize()
	local _, instanceType, diff = GetInstanceInfo()
	if instanceType == "party" or instanceType == "arena" then
		return 5
	elseif instanceType == "none" or instanceType == "pvp" or diff == 9 then
		return 40
	elseif diff == 3 or diff == 5 then
		return 10
	elseif diff == 4 or diff == 6 or diff == 7 then
		return 25
	end
end

do -- BigBrother:SendChatMessageList(title, list, channel)
	local chatTypes = {
		SELF = true,
		SAY = true,
		GROUP = true,
		PARTY = true,
		GUILD = true,
		OFFICIER = true,
		RAID = true,
		RAID_WARNING = true,
		WHISPER = true,
		CHANNEL = true,
	}

	local function outgoingFilter(self, event, msg)
		if msg:sub(1, 11) == "BigBrother>" then return true end
	end
	ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", outgoingFilter)

	--pick the right output or fallback to self
	local function getChannel(channel)
		channel = channel and channel:upper() or BigBrother.db.profile.outputDefault:upper()

		--seems like a dirty way to do this ;[ makes the syntax /bb qc channel:mychannel or /run BigBrother:ConsumableCheck("channel:mychannel")
		local channelName
		channel, channelName = strsplit(":", channel, 2)

		if not chatTypes[channel] then
			return "SELF"
		end

		if channel == "CHANNEL" then
			local index = GetChannelName(channelName)
			if index > 0 then
				return channel, index
			else
				return "SELF"
			end
		elseif channel == "WHISPER" then
			if UnitCanCooperate("player", "target") then
				return channel, UnitName("target")
			else
				return "SELF"
			end
		elseif channel == "GROUP" then
			if IsInRaid() then
				channel = "RAID"
			elseif IsInGroup() then
				channel = "PARTY"
			else
				return "SELF"
			end
		end
		if channel == "RAID" or channel == "PARTY" then
			if IsPartyLFG() then
				return "INSTANCE_CHAT"
			elseif not IsInGroup() then
				return "SELF"
			end
		end

		return channel
	end

	function BigBrother:SendChatMessage(text, channel)
		if type(text) ~= "string" or text == "" then return end

		local target
		channel, target = getChannel(channel)
		if not channel then return end

		if channel == "SELF" then
			print(format("|Hbigbrother:%s|h|cff33ff99BigBrother|r|h: %s", text, text))
		else
			SendChatMessage(text, channel, nil, target)
		end
	end

	function BigBrother:SendChatMessageList(title, list, channel)
		if type(title) ~= "string" or title == "" then return end
		if type(list) ~= "table" or #list == 0 then return end

		local target
		channel, target = getChannel(channel)
		if not channel then return end

		table.sort(list)
		local names = table.concat(list, ", ")
		local text = ("%s (%d): %s"):format(title, #list, names)

		if channel == "SELF" then
			local coloredText = ("|cffff8040%s (%d)|r: %s"):format(title, #list, names)
			print(format("|Hbigbrother:%s|h|cff33ff99BigBrother|r|h: %s", text, coloredText))
		else
			SendChatMessage(text, channel, nil, target)
		end
	end
end

do -- BigBrother:GetCheckedGroups()
	local fiveManGroups = { true }
	local tenManGroups = { true, true }
	local twentyFiveManGroups = { true, true, true, true, true }
	local allGroups = { true, true, true, true, true, true, true, true }

	function BigBrother:GetCheckedGroups()
		if self.db.profile.autogroups then
			local size = self:GetInstanceSize()
			if size == 25 then
				return twentyFiveManGroups
			elseif size == 10 then
				return tenManGroups
			elseif size == 5 then
				return fiveManGroups
			else
				return allGroups
			end
		else
			return self.db.profile.groups
		end
	end
end

do -- BigBrother:GetChatChannels()
	local channels = {}
	local function updateChannels()
		wipe(channels)
		for i = 1, GetNumDisplayChannels() do
			local name, _, _, _, _, _, category = GetChannelDisplayInfo(i)
			if category == "CHANNEL_CATEGORY_CUSTOM" then
				channels[name] = name
			end
		end
	end

	function BigBrother:GetChatChannels()
		return channels
	end

	function BigBrother:CHANNEL_UI_UPDATE()
		updateChannels()
		LibStub("AceConfigRegistry-3.0"):NotifyChange("BigBrother")
	end
end

do --BigBrother:IterateGroup()
	local partyUnits = {"player"}
	for i = 1, MAX_PARTY_MEMBERS do
		partyUnits[#partyUnits + 1] = ("party%d"):format(i)
	end

	local raidUnits = {}
	for i = 1, MAX_RAID_MEMBERS do
		raidUnits[#raidUnits + 1] = ("raid%d"):format(i)
	end

	function BigBrother:IterateGroup()
		local i = 0
		local function iter(a)
			i = i + 1
			return a[i], i
		end

		return iter, (IsInRaid() and raidUnits or partyUnits)
	end
end

function BigBrother:BuffCheck(channel)
	self:ConsumableCheck(channel)
	self:RaidBuffCheck(channel)
end

function BigBrother:OpenOptions()
	InterfaceOptionsFrame_OpenToCategory("BigBrother")
end

SlashCmdList.BIGBROTHER = function(input)
	local command, rest = string.match(input, "^(%S*)%s*(.-)$")
	command = command and command:lower()
	if command == "check" or command == "qc" then
		BigBrother:BuffCheck(rest ~= "" and rest)
	elseif command == "window" or command == "bc" then
		BigBrother:ToggleBuffWindow()
	elseif command == "config" then
		BigBrother:OpenOptions()
	else
		BigBrother:Print("|cffffff7f/bb check [self||group||party||raid||guild||officer]|r: " .. L["Report who is missing consumables and/or buffs."])
		BigBrother:Print("|cffffff7f/bb window|r: " .. L["Opens the buff window (drag the bottom to resize)."])
		BigBrother:Print("|cffffff7f/bb config|r: " .. L["Open settings."])
	end
end
SLASH_BIGBROTHER1 = "/bigbrother"
SLASH_BIGBROTHER2 = "/bb"
